use Test::More;

use_ok 'Lingy';
use_ok 'Lingy::CLI';
use_ok 'Lingy::Common';
use_ok 'Lingy::Env';
use_ok 'Lingy::Eval';
use_ok 'Lingy::NS';
use_ok 'Lingy::Printer';
use_ok 'Lingy::ReadLine';
use_ok 'Lingy::Reader';
use_ok 'Lingy::RT';
use_ok 'Lingy::Test';
use_ok 'Lingy::Types';

use_ok 'Lingy::Core';
use_ok 'Lingy::Lang::Numbers';
use_ok 'Lingy::Lang::Util';

done_testing;
